@@ -35,7 +35,6 @@
 #include <fcntl.h>
 #include <ctype.h>
 #include <errno.h>
-#include <netinet/in.h>
 #include <sys/asoundlib.h>
 #include "aconfig.h"
 #include "formats.h"
@@ -199,46 +198,46 @@ static void device_list(void)
 
 	mask = snd_cards_mask();
 	if (!mask) {
-		printf("%s: no soundcards found...\n", command);
+		fprintf(stderr, "%s: no soundcards found...\n", command);
 		return;
 	}
 	for (card = 0; card < SND_CARDS; card++) {
 		if (!(mask & (1 << card)))
 			continue;
 		if ((err = snd_ctl_open(&handle, card)) < 0) {
-			printf("Error: control open (%i): %s\n", card, snd_strerror(err));
+			fprintf(stderr, "Error: control open (%i): %s\n", card, snd_strerror(err));
 			continue;
 		}
 		if ((err = snd_ctl_hw_info(handle, &info)) < 0) {
-			printf("Error: control hardware info (%i): %s\n", card, snd_strerror(err));
+			fprintf(stderr, "Error: control hardware info (%i): %s\n", card, snd_strerror(err));
 			snd_ctl_close(handle);
 			continue;
 		}
 		for (dev = 0; dev < info.pcmdevs; dev++) {
 			if ((err = snd_ctl_pcm_info(handle, dev, &pcminfo)) < 0) {
-				printf("Error: control digital audio info (%i): %s\n", card, snd_strerror(err));
+				fprintf(stderr, "Error: control digital audio info (%i): %s\n", card, snd_strerror(err));
 				continue;
 			}
-			printf("%s: %i [%s] / #%i: %s\n",
+			fprintf(stderr, "%s: %i [%s] / #%i: %s\n",
 			       info.name,
 			       card + 1,
 			       info.id,
 			       dev,
 			       pcminfo.name);
-			printf("  Directions: %s%s%s\n",
+			fprintf(stderr, "  Directions: %s%s%s\n",
 			       pcminfo.flags & SND_PCM_INFO_PLAYBACK ? "playback " : "",
 			       pcminfo.flags & SND_PCM_INFO_CAPTURE ? "capture " : "",
 			       pcminfo.flags & SND_PCM_INFO_DUPLEX ? "duplex " : "");
-			printf("  Playback subdevices: %i\n", pcminfo.playback + 1);
-			printf("  Capture subdevices: %i\n", pcminfo.capture + 1);
+			fprintf(stderr, "  Playback subdevices: %i\n", pcminfo.playback + 1);
+			fprintf(stderr, "  Capture subdevices: %i\n", pcminfo.capture + 1);
 			if (pcminfo.flags & SND_PCM_INFO_PLAYBACK) {
 				for (idx = 0; idx <= pcminfo.playback; idx++) {
 					memset(&chninfo, 0, sizeof(chninfo));
 					chninfo.channel = SND_PCM_CHANNEL_PLAYBACK;
 					if ((err = snd_ctl_pcm_channel_info(handle, dev, SND_PCM_CHANNEL_PLAYBACK, idx, &chninfo)) < 0) {
-						printf("Error: control digital audio playback info (%i): %s\n", card, snd_strerror(err));
+						fprintf(stderr, "Error: control digital audio playback info (%i): %s\n", card, snd_strerror(err));
 					} else {
-						printf("  Playback subdevice #%i: %s\n", idx, chninfo.subname);
+						fprintf(stderr, "  Playback subdevice #%i: %s\n", idx, chninfo.subname);
 					}
 				}
 			}
@@ -247,9 +246,9 @@ static void device_list(void)
 					memset(&chninfo, 0, sizeof(chninfo));
 					chninfo.channel = SND_PCM_CHANNEL_CAPTURE;
 					if ((err = snd_ctl_pcm_channel_info(handle, dev, SND_PCM_CHANNEL_CAPTURE, 0, &chninfo)) < 0) {
-						printf("Error: control digital audio capture info (%i): %s\n", card, snd_strerror(err));
+						fprintf(stderr, "Error: control digital audio capture info (%i): %s\n", card, snd_strerror(err));
 					} else {
-						printf("  Capture subdevice #%i: %s\n", idx, chninfo.subname);
+						fprintf(stderr, "  Capture subdevice #%i: %s\n", idx, chninfo.subname);
 					}
 				}
 			}
@@ -484,42 +483,61 @@ static int test_vocfile(void *buffer)
 }
 
 /*
- * test, if it's a .WAV file, 0 if ok (and set the speed, stereo etc.)
- *                            < 0 if not
+ * test, if it's a .WAV file, > 0 if ok (and set the speed, stereo etc.)
+ *                            == 0 if not
+ * Value returned is bytes to be discarded.
  */
-static int test_wavefile(void *buffer)
+static int test_wavefile(void *buffer, size_t size)
 {
-	WaveHeader *wp = buffer;
+	WaveHeader *h = buffer;
+	WaveFmtHeader *f;
+	WaveChunkHeader *c;
 
-	if (wp->main_chunk == WAV_RIFF && wp->chunk_type == WAV_WAVE &&
-	    wp->sub_chunk == WAV_FMT && wp->data_chunk == WAV_DATA) {
-		if (wp->format != WAV_PCM_CODE) {
-			fprintf(stderr, "%s: can't play not PCM-coded WAVE-files\n", command);
-			exit(1);
-		}
-		if (wp->modus < 1 || wp->modus > 32) {
-			fprintf(stderr, "%s: can't play WAVE-files with %d tracks\n",
-				command, wp->modus);
-			exit(1);
-		}
-		format.voices = wp->modus;
-		switch (wp->bit_p_spl) {
-		case 8:
-			format.format = SND_PCM_SFMT_U8;
-			break;
-		case 16:
-			format.format = SND_PCM_SFMT_S16_LE;
-			break;
-		default:
-			fprintf(stderr, "%s: can't play WAVE-files with sample %d bits wide\n",
-				command, wp->bit_p_spl);
-		}
-		format.rate = wp->sample_fq;
-		count = wp->data_length;
-		check_new_format(&format);
+	if (h->magic != WAV_RIFF || h->type != WAV_WAVE)
 		return 0;
+	c = (WaveChunkHeader*)((char *)buffer + sizeof(WaveHeader));
+	while (c->type != WAV_FMT) {
+		c = (WaveChunkHeader*)((char*)c + sizeof(*c) + LE_INT(c->length));
+		if ((char *)c + sizeof(*c) > (char*) buffer + size) {
+			fprintf(stderr, "%s: cannot found WAVE fmt chunk\n", command);
+			exit(1);
+		}
 	}
-	return -1;
+	f = (WaveFmtHeader*) c;
+	if (LE_SHORT(f->format) != WAV_PCM_CODE) {
+		fprintf(stderr, "%s: can't play not PCM-coded WAVE-files\n", command);
+		exit(1);
+	}
+	if (LE_SHORT(f->modus) < 1) {
+		fprintf(stderr, "%s: can't play WAVE-files with %d tracks\n",
+			command, LE_SHORT(f->modus));
+		exit(1);
+	}
+	format.voices = LE_SHORT(f->modus);
+	switch (LE_SHORT(f->bit_p_spl)) {
+	case 8:
+		format.format = SND_PCM_SFMT_U8;
+		break;
+	case 16:
+		format.format = SND_PCM_SFMT_S16_LE;
+		break;
+	default:
+		fprintf(stderr, "%s: can't play WAVE-files with sample %d bits wide\n",
+			command, LE_SHORT(f->bit_p_spl));
+		exit(1);
+	}
+	format.rate = LE_INT(f->sample_fq);
+	while (c->type != WAV_DATA) {
+		c = (WaveChunkHeader*)((char*)c + sizeof(*c) + LE_INT(c->length));
+		if ((char *)c + sizeof(*c) > (char*) buffer + size) {
+			fprintf(stderr, "%s: cannot found WAVE data chunk\n", command);
+			exit(1);
+		}
+	}
+
+	count = LE_INT(c->length);
+	check_new_format(&format);
+	return (char *)c + sizeof(*c) - (char *) buffer;
 }
 
 /*
@@ -530,12 +548,12 @@ static int test_au(int fd, void *buffer)
 {
 	AuHeader *ap = buffer;
 
-	if (ntohl(ap->magic) != AU_MAGIC)
+	if (ap->magic != AU_MAGIC)
 		return -1;
-	if (ntohl(ap->hdr_size) > 128 || ntohl(ap->hdr_size) < 24)
+	if (BE_INT(ap->hdr_size) > 128 || BE_INT(ap->hdr_size) < 24)
 		return -1;
-	count = ntohl(ap->data_size);
-	switch (ntohl(ap->encoding)) {
+	count = BE_INT(ap->data_size);
+	switch (BE_INT(ap->encoding)) {
 	case AU_FMT_ULAW:
 		format.format = SND_PCM_SFMT_MU_LAW;
 		break;
@@ -548,13 +566,13 @@ static int test_au(int fd, void *buffer)
 	default:
 		return -1;
 	}
-	format.rate = ntohl(ap->sample_rate);
+	format.rate = BE_INT(ap->sample_rate);
 	if (format.rate < 2000 || format.rate > 256000)
 		return -1;
-	format.voices = ntohl(ap->channels);
+	format.voices = BE_INT(ap->channels);
 	if (format.voices < 1 || format.voices > 128)
 		return -1;
-	if (read(fd, buffer + sizeof(AuHeader), ntohl(ap->hdr_size) - sizeof(AuHeader)) < 0) {
+	if (read(fd, buffer + sizeof(AuHeader), BE_INT(ap->hdr_size) - sizeof(AuHeader)) < 0) {
 		fprintf(stderr, "%s: read error\n", command);
 		exit(1);
 	}
@@ -1004,8 +1022,12 @@ static void begin_voc(int fd, u_long cnt)
 /* write a WAVE-header */
 static void begin_wave(int fd, u_long cnt)
 {
-	WaveHeader wh;
+	WaveHeader h;
+	WaveFmtHeader f;
+	WaveChunkHeader c;
 	int bits;
+	u_int tmp;
+	u_short tmp2;
 
 	bits = 8;
 	switch (format.format) {
@@ -1019,25 +1041,35 @@ static void begin_wave(int fd, u_long cnt)
 		fprintf(stderr, "%s: Wave doesn't support %s format...\n", command, snd_pcm_get_format_name(format.format));
 		exit(1);
 	}
-	wh.main_chunk = WAV_RIFF;
-	wh.length = cnt + sizeof(WaveHeader) - 8;
-	wh.chunk_type = WAV_WAVE;
-	wh.sub_chunk = WAV_FMT;
-	wh.sc_len = 16;
-	wh.format = WAV_PCM_CODE;
-	wh.modus = format.voices;
-	wh.sample_fq = format.rate;
+	h.magic = WAV_RIFF;
+	tmp = cnt + sizeof(WaveHeader) + sizeof(WaveFmtHeader) + sizeof(WaveChunkHeader) - 8;
+	h.length = LE_INT(tmp);
+	h.type = WAV_WAVE;
+
+	f.type = WAV_FMT;
+	f.length = LE_INT(16);
+	f.format = LE_INT(WAV_PCM_CODE);
+	f.modus = LE_SHORT(format.voices);
+	f.sample_fq = LE_INT(format.rate);
 #if 0
-	wh.byte_p_spl = (samplesize == 8) ? 1 : 2;
-	wh.byte_p_sec = dsp_speed * wh.modus * wh.byte_p_spl;
+	tmp2 = (samplesize == 8) ? 1 : 2;
+	f.byte_p_spl = LE_SHORT(tmp2);
+	tmp2 = dsp_speed * format.voices * tmp2;
+	f.byte_p_sec = LE_SHORT(tmp2);
 #else
-	wh.byte_p_spl = wh.modus * ((bits + 7) / 8);
-	wh.byte_p_sec = wh.byte_p_spl * format.rate;
+	tmp2 = format.voices * ((bits + 7) / 8);
+	f.byte_p_spl = LE_SHORT(tmp2);
+	tmp2 = tmp2 * format.rate;
+	f.byte_p_sec = LE_SHORT(tmp2);
 #endif
-	wh.bit_p_spl = bits;
-	wh.data_chunk = WAV_DATA;
-	wh.data_length = cnt;
-	if (write(fd, &wh, sizeof(WaveHeader)) != sizeof(WaveHeader)) {
+	f.bit_p_spl = LE_SHORT(bits);
+
+	c.type = WAV_DATA;
+	c.length = LE_INT(cnt);
+
+	if (write(fd, &h, sizeof(WaveHeader)) != sizeof(WaveHeader) ||
+	    write(fd, &f, sizeof(WaveFmtHeader)) != sizeof(WaveFmtHeader) ||
+	    write(fd, &c, sizeof(WaveChunkHeader)) != sizeof(WaveChunkHeader)) {
 		fprintf(stderr, "%s: write error\n", command);
 		exit(1);
 	}
@@ -1048,25 +1080,25 @@ static void begin_au(int fd, u_long cnt)
 {
 	AuHeader ah;
 
-	ah.magic = htonl(AU_MAGIC);
-	ah.hdr_size = htonl(24);
-	ah.data_size = htonl(cnt);
+	ah.magic = AU_MAGIC;
+	ah.hdr_size = BE_INT(24);
+	ah.data_size = BE_INT(cnt);
 	switch (format.format) {
 	case SND_PCM_SFMT_MU_LAW:
-		ah.encoding = htonl(AU_FMT_ULAW);
+		ah.encoding = BE_INT(AU_FMT_ULAW);
 		break;
 	case SND_PCM_SFMT_U8:
-		ah.encoding = htonl(AU_FMT_LIN8);
+		ah.encoding = BE_INT(AU_FMT_LIN8);
 		break;
 	case SND_PCM_SFMT_S16_LE:
-		ah.encoding = htonl(AU_FMT_LIN16);
+		ah.encoding = BE_INT(AU_FMT_LIN16);
 		break;
 	default:
 		fprintf(stderr, "%s: Sparc Audio doesn't support %s format...\n", command, snd_pcm_get_format_name(format.format));
 		exit(1);
 	}
-	ah.sample_rate = htonl(format.rate);
-	ah.channels = htonl(format.voices);
+	ah.sample_rate = BE_INT(format.rate);
+	ah.channels = BE_INT(format.voices);
 	if (write(fd, &ah, sizeof(AuHeader)) != sizeof(AuHeader)) {
 		fprintf(stderr, "%s: write error\n", command);
 		exit(1);
@@ -1367,21 +1399,22 @@ static void playback(char *name)
 		voc_play(fd, ofs, name);
 		goto __end;
 	}
-	/* read bytes for WAVE-header */
+	/* read 64 bytes for WAVE-header */
 	if (read(fd, audiobuf + sizeof(VocHeader),
-		 sizeof(WaveHeader) - sizeof(VocHeader)) !=
-	    sizeof(WaveHeader) - sizeof(VocHeader)) {
+		 64 - sizeof(VocHeader)) !=
+	    64 - sizeof(VocHeader)) {
 		fprintf(stderr, "%s: read error", command);
 		exit(1);
 	}
-	if (test_wavefile(audiobuf) >= 0) {
-		playback_go(fd, 0, count, FORMAT_WAVE, name);
+	if ((ofs = test_wavefile(audiobuf, 64)) > 0) {
+		memmove(audiobuf, audiobuf + ofs, 64 - ofs);
+		playback_go(fd, 64 - ofs, count, FORMAT_WAVE, name);
 	} else {
 		/* should be raw data */
 		check_new_format(&rformat);
 		init_raw_data();
 		count = calc_count();
-		playback_go(fd, sizeof(WaveHeader), count, FORMAT_RAW, name);
+		playback_go(fd, 64, count, FORMAT_RAW, name);
 	}
       __end:
 	if (fd != 0)
